@@ -22,6 +22,12 @@
 #include "wlr-screencopy-unstable-v1-client-protocol.h"
 #include "xdg-output-unstable-v1-client-protocol.h"
 
+#include "config.h"
+
+#ifdef HAVE_OPENCL
+std::unique_ptr<OpenCL> opencl;
+#endif
+
 std::mutex frame_writer_mutex, frame_writer_pending_mutex;
 std::unique_ptr<FrameWriter> frame_writer;
 
@@ -308,6 +314,14 @@ static void write_loop(FrameWriterParams params, PulseReaderParams pulseParams)
             params.height = buffer.height;
             frame_writer = std::unique_ptr<FrameWriter> (new FrameWriter(params));
 
+#ifdef HAVE_OPENCL
+            if (params.opencl && params.force_yuv)
+            {
+                frame_writer->opencl = std::move(opencl);
+                frame_writer->opencl->init(params.width, params.height);
+            }
+#endif
+
             if (params.enable_audio)
             {
                 pulseParams.audio_frame_size = frame_writer->get_audio_buffer_size();
@@ -479,13 +493,100 @@ static wf_recorder_output* detect_output_from_region(const capture_region& regio
     return nullptr;
 }
 
+static void help()
+{
+    printf(R"(Usage: wf-recorder [OPTION]... -f [FILE]...
+Screen recording of wlroots-based compositors
+
+With no FILE, start recording the current screen.
+
+  -a, --audio [DEVICE]      Starts recording the screen with audio.
+                            [DEVICE] argument is optional.
+                            In case you want to specify the pulseaudio device which will capture
+                            the audio, you can run this command with the name of that device.
+                            You can find your device by running: pactl list sinks | grep Name
+
+  -c, --codec               Specifies the codec of the video. Supports  GIF output also.
+                            To modify codec parameters, use -p <option_name>=<option_value>
+
+  -d, --device              Selects the device to use when encoding the video
+                            Some drivers report support for rgb0 data for vaapi input but
+                            really only support yuv.
+
+  -f <filename>.ext         By using the -f option the output file will have the name :
+                            filename.ext and the file format will be determined by provided
+                            while extension .ext . If the extension .ext provided is not
+                            recognized by your FFmpeg muxers, the command will fail.
+                            You can check the muxers that your FFmpeg installation supports by
+                            running  : ffmpeg -muxers
+
+  -m, --muxer               Set the output format to a specific muxer instead of detecting it
+                            from the filename.
+
+  -x, --pixel-format        Set the output pixel format. These can be found by running:
+                            *ffmpeg -pix_fmts*
+
+  -g, --geometry            Selects a specific part of the screen.
+
+  -h, --help                Prints this help screen.
+
+  -l, --log                 Generates a log on the current terminal. Debug purposes.
+
+  -o, --output              Specify the output where the video is to be recorded.
+
+  -p, --codec-param         Change the codec parameters.
+                            -p <option_name>=<option_value>)");
+#ifdef HAVE_OPENCL
+    printf(R"(
+
+  -e, --opencl              Use the -e[#] or --opencl[=#] in conjunction with -t or --force-yuv option
+                            to use opencl for gpu accelerated conversion of data to yuv. # is one
+                            of the devices listed when running without specifying #.)");
+#endif
+    printf(R"(
+
+  -t, --force-yuv           Use the -t or --force-yuv option to force conversion of the data to
+                            yuv format, before sending it to the gpu.
+
+  -b, --bframes             This option is used to set the maximum number of b-frames to be used.
+                            If b-frames are not supported by your hardware, set this to 0.)" "\n\n" R"(
+Examples:
+
+  Video Only:
+
+  - wf-recorder                         Records the video. Use Ctrl+C to stop recording.
+                                        The video file will be stored as recording.mp4 in the
+                                        current working directory.
+
+  - wf-recorder -f <filename>.ext       Records the video. Use Ctrl+C to stop recording.
+                                        The video file will be stored as <outputname>.ext in the
+                                        current working directory.
+
+  Video and Audio:
+
+  - wf-recorder -a                      Records the audio. Use Ctrl+C to stop recording.
+                                        The video file will be stored as recording.mp4 in the
+                                        current working directory.
+
+  - wf-recorder -a -f <filename>.ext    Records the audio. Use Ctrl+C to stop recording.
+                                        The video file will be stored as <outputname>.ext in the
+                                        current working directory.
+
+)" "\n");
+    exit(EXIT_SUCCESS);
+}
+
 int main(int argc, char *argv[])
 {
     FrameWriterParams params;
     params.file = "recording.mp4";
-    params.codec = "libx264";
+    params.codec = DEFAULT_CODEC;
     params.enable_ffmpeg_debug_output = false;
     params.enable_audio = false;
+    params.force_yuv = false;
+    params.opencl = false;
+    params.opencl_device = -1;
+    params.bframes = -1;
 
     PulseReaderParams pulseParams;
 
@@ -497,19 +598,25 @@ int main(int argc, char *argv[])
     struct option opts[] = {
         { "output",          required_argument, NULL, 'o' },
         { "file",            required_argument, NULL, 'f' },
+        { "muxer",           required_argument, NULL, 'm' },
+        { "pixel-format",    required_argument, NULL, 'x' },
         { "geometry",        required_argument, NULL, 'g' },
         { "codec",           required_argument, NULL, 'c' },
         { "codec-param",     required_argument, NULL, 'p' },
         { "device",          required_argument, NULL, 'd' },
         { "log",             no_argument,       NULL, 'l' },
         { "audio",           optional_argument, NULL, 'a' },
+        { "help",            no_argument,       NULL, 'h' },
+        { "force-yuv",       no_argument,       NULL, 't' },
+        { "opencl",          optional_argument, NULL, 'e' },
+        { "bframes",         optional_argument, NULL, 'b' },
         { 0,                 0,                 NULL,  0  }
     };
 
     int c, i;
     std::string param;
     size_t pos;
-    while((c = getopt_long(argc, argv, "o:f:g:c:p:d:la::", opts, &i)) != -1)
+    while((c = getopt_long(argc, argv, "o:f:m:x:g:c:p:d:b:la::te::h", opts, &i)) != -1)
     {
         switch(c)
         {
@@ -521,6 +628,14 @@ int main(int argc, char *argv[])
                 cmdline_output = optarg;
                 break;
 
+            case 'm':
+                params.muxer = optarg;
+                break;
+
+            case 'x':
+                params.pix_fmt = optarg;
+                break;
+
             case 'g':
                 selected_region.set_from_string(optarg);
                 break;
@@ -533,6 +648,10 @@ int main(int argc, char *argv[])
                 params.hw_device = optarg;
                 break;
 
+            case 'b':
+                params.bframes = optarg ? atoi(optarg) : -1;
+                break;
+
             case 'l':
                 params.enable_ffmpeg_debug_output = true;
                 break;
@@ -542,6 +661,19 @@ int main(int argc, char *argv[])
                 pulseParams.audio_source = optarg ? strdup(optarg) : NULL;
                 break;
 
+            case 't':
+                params.force_yuv = true;
+                break;
+
+            case 'h':
+                help();
+                break;
+
+            case 'e':
+                params.opencl = true;
+                params.opencl_device = optarg ? atoi(optarg) : -1;
+                break;
+
             case 'p':
                 param = optarg;
                 pos = param.find("=");
@@ -562,7 +694,8 @@ int main(int argc, char *argv[])
     }
 
     display = wl_display_connect(NULL);
-    if (display == NULL) {
+    if (display == NULL) 
+    {
         fprintf(stderr, "failed to create display: %m\n");
         return EXIT_FAILURE;
     }
@@ -633,6 +766,11 @@ int main(int argc, char *argv[])
 
     printf("selected region %d %d %d %d\n", selected_region.x, selected_region.y, selected_region.width, selected_region.height);
 
+#ifdef HAVE_OPENCL
+     if (params.opencl && params.force_yuv)
+         opencl = std::unique_ptr<OpenCL> (new OpenCL(params.opencl_device));
+#endif
+
     timespec first_frame;
     first_frame.tv_sec = -1;
 
